@@ -1183,3 +1183,161 @@ fn test_mutable_args_once() {
     assert_eq!((2, 2), mutable_args_once(1, 1));
     assert_eq!((2, 2), mutable_args_once(5, 6));
 }
+
+#[cfg(feature = "redis_store")]
+mod redis_tests {
+    use super::*;
+    use cached::proc_macro::io_cached;
+    use cached::RedisCache;
+    use thiserror::Error;
+
+    #[derive(Error, Debug, PartialEq, Clone)]
+    enum TestError {
+        #[error("error with redis cache `{0}`")]
+        RedisError(String),
+        #[error("count `{0}`")]
+        Count(u32),
+    }
+
+    #[io_cached(
+        redis = true,
+        time = 1,
+        cache_prefix_block = "{ \"__cached_redis_proc_macro_test_fn_cached_redis\" }",
+        map_error = r##"|e| TestError::RedisError(format!("{:?}", e))"##
+    )]
+    fn cached_redis(n: u32) -> Result<u32, TestError> {
+        if n < 5 {
+            Ok(n)
+        } else {
+            Err(TestError::Count(n))
+        }
+    }
+
+    #[test]
+    fn test_cached_redis() {
+        assert_eq!(cached_redis(1), Ok(1));
+        assert_eq!(cached_redis(1), Ok(1));
+        assert_eq!(cached_redis(5), Err(TestError::Count(5)));
+        assert_eq!(cached_redis(6), Err(TestError::Count(6)));
+    }
+
+    #[io_cached(
+        redis = true,
+        time = 1,
+        with_cached_flag = true,
+        map_error = r##"|e| TestError::RedisError(format!("{:?}", e))"##
+    )]
+    fn cached_redis_cached_flag(n: u32) -> Result<cached::Return<u32>, TestError> {
+        if n < 5 {
+            Ok(cached::Return::new(n))
+        } else {
+            Err(TestError::Count(n))
+        }
+    }
+
+    #[test]
+    fn test_cached_redis_cached_flag() {
+        assert!(!cached_redis_cached_flag(1).unwrap().was_cached);
+        assert!(cached_redis_cached_flag(1).unwrap().was_cached);
+        assert!(cached_redis_cached_flag(5).is_err());
+        assert!(cached_redis_cached_flag(6).is_err());
+    }
+
+    #[io_cached(
+        map_error = r##"|e| TestError::RedisError(format!("{:?}", e))"##,
+        type = "cached::RedisCache<u32, u32>",
+        create = r##" { RedisCache::new("cache_redis_test_cache_create", 1).set_refresh(true).build().expect("error building redis cache") } "##
+    )]
+    fn cached_redis_cache_create(n: u32) -> Result<u32, TestError> {
+        if n < 5 {
+            Ok(n)
+        } else {
+            Err(TestError::Count(n))
+        }
+    }
+
+    #[test]
+    fn test_cached_redis_cache_create() {
+        assert_eq!(cached_redis_cache_create(1), Ok(1));
+        assert_eq!(cached_redis_cache_create(1), Ok(1));
+        assert_eq!(cached_redis_cache_create(5), Err(TestError::Count(5)));
+        assert_eq!(cached_redis_cache_create(6), Err(TestError::Count(6)));
+    }
+
+    #[cfg(any(feature = "redis_async_std", feature = "redis_tokio"))]
+    mod async_redis_tests {
+        use super::*;
+
+        #[io_cached(
+            redis = true,
+            time = 1,
+            cache_prefix_block = "{ \"__cached_redis_proc_macro_test_fn_async_cached_redis\" }",
+            map_error = r##"|e| TestError::RedisError(format!("{:?}", e))"##
+        )]
+        async fn async_cached_redis(n: u32) -> Result<u32, TestError> {
+            if n < 5 {
+                Ok(n)
+            } else {
+                Err(TestError::Count(n))
+            }
+        }
+
+        #[async_std::test]
+        async fn test_async_cached_redis() {
+            assert_eq!(async_cached_redis(1).await, Ok(1));
+            assert_eq!(async_cached_redis(1).await, Ok(1));
+            assert_eq!(async_cached_redis(5).await, Err(TestError::Count(5)));
+            assert_eq!(async_cached_redis(6).await, Err(TestError::Count(6)));
+        }
+
+        #[io_cached(
+            redis = true,
+            time = 1,
+            with_cached_flag = true,
+            map_error = r##"|e| TestError::RedisError(format!("{:?}", e))"##
+        )]
+        async fn async_cached_redis_cached_flag(n: u32) -> Result<cached::Return<u32>, TestError> {
+            if n < 5 {
+                Ok(cached::Return::new(n))
+            } else {
+                Err(TestError::Count(n))
+            }
+        }
+
+        #[async_std::test]
+        async fn test_async_cached_redis_cached_flag() {
+            assert!(!async_cached_redis_cached_flag(1).await.unwrap().was_cached);
+            assert!(async_cached_redis_cached_flag(1).await.unwrap().was_cached,);
+            assert!(async_cached_redis_cached_flag(5).await.is_err());
+            assert!(async_cached_redis_cached_flag(6).await.is_err());
+        }
+
+        use cached::AsyncRedisCache;
+        #[io_cached(
+            map_error = r##"|e| TestError::RedisError(format!("{:?}", e))"##,
+            type = "cached::AsyncRedisCache<u32, u32>",
+            create = r##" { AsyncRedisCache::new("async_cached_redis_test_cache_create", 1).set_refresh(true).build().await.expect("error building async redis cache") } "##
+        )]
+        async fn async_cached_redis_cache_create(n: u32) -> Result<u32, TestError> {
+            if n < 5 {
+                Ok(n)
+            } else {
+                Err(TestError::Count(n))
+            }
+        }
+
+        #[async_std::test]
+        async fn test_async_cached_redis_cache_create() {
+            assert_eq!(async_cached_redis_cache_create(1).await, Ok(1));
+            assert_eq!(async_cached_redis_cache_create(1).await, Ok(1));
+            assert_eq!(
+                async_cached_redis_cache_create(5).await,
+                Err(TestError::Count(5))
+            );
+            assert_eq!(
+                async_cached_redis_cache_create(6).await,
+                Err(TestError::Count(6))
+            );
+        }
+    }
+}
